#!/bin/python3
#%%
import numpy as np
import pandas as pd
from scipy.stats import spearmanr
import matplotlib.pyplot as plt
import matplotlib.font_manager as fm


font_path = './tools/tables/SimHei.ttf'
fontproperties = fm.FontProperties(fname=font_path)


def plot_corr(
        xs: pd.Series, 
        ys: pd.Series = None, 
        **params):

    if isinstance(xs, dict) and ys is None:
        d = xs
        keys = list(d)
        xs = d[keys[0]]
        ys = d[keys[1]]
        if 'xlabel' not in params:
            params['xlabel'] = keys[0]
        if 'ylabel' not in params:
            params['ylabel'] = keys[1]

    xlabel = params.get('xlabel', 'x')
    ylabel = params.get('ylabel', 'y')
    title = params.get('title', 'title')
    diag = params.get('diag', True)
    alpha = params.get('alpha', .5)


    idx = (~xs.isna()) & (~ys.isna())

    x = xs[idx]
    y = ys[idx]

    if diag:
        m0 = min(x.min(), y.min())
        m1 = max(x.max(), y.max())
        plt.plot([m0, m1], [m0, m1], 'C7--', alpha = .5)

    plt.plot(x, y, 'C0.', alpha = alpha)
    plt.xticks(fontproperties=fontproperties)
    plt.yticks(fontproperties=fontproperties)
    plt.grid(True)
    plt.xlabel(xlabel=xlabel, fontproperties=fontproperties)
    plt.ylabel(ylabel=ylabel, fontproperties=fontproperties)
    plt.title(title, fontproperties=fontproperties, size = 12)

    plt.show()

    xn0, xn1 = len(xs), len(xs) - xs.isna().sum()
    yn0, yn1 = len(ys), len(ys) - ys.isna().sum()
    m = len(x)

    print(f'x.coverage = {xn1}/{xn0} = {xn1/xn0}')
    print(f'y.coverage = {yn1}/{yn0} = {yn1/yn0}')
    print(f'x.intersec = {m}/{xn0} = {m/xn0}')
    print(f'y.intersec = {m}/{yn0} = {m/yn0}')

    print(f'pearson = {x.corr(y)}')
    print(f'spearmanr = {spearmanr(x, y)[0]}')

    return None


def plot_hist(data: dict, **params):

    if isinstance(data, pd.Series):
        data = {data.name: data}

    elif isinstance(data, dict):
        data = {k: pd.Series(v) for k, v in data.items()}

    elif isinstance(data, pd.DataFrame):
        data = {k: data[k] for k in sorted(data.columns)}

    title = params.get('title', 'title')
    xlabel = params.get('xlabel', 'x')
    ylabel = params.get('ylabel', 'y')
    alpha = params.get('alpha', .5)
    bins = params.get('bins', 20)
    loc = params.get('loc', 'upper right')
    value_range = params.get('value_range', None)
    legend_alpha = params.get('legend_alpha', .5)
    drop_outliers = params.get('drop_outliers', None)

    if value_range is None:
        if drop_outliers is None:
            m0 = min([sr.min() for sr in data.values()])
            m1 = max([sr.max() for sr in data.values()])
        else:
            m0 = min([sr.quantile(drop_outliers) for sr in data.values()])
            m1 = max([sr.quantile(1-drop_outliers) for sr in data.values()])
    else:
        m0, m1 = value_range

    print(f'value_range = [{m0}, {m1}]\n')

    for idx, (label, sr) in enumerate(data.items(), start=1):
        color=f'C{idx}'
        plt.hist(
            sr, 
            density=True, 
            alpha = alpha, 
            bins = bins, 
            range=[m0, m1],
            label = label, 
            color=color
            )
        x_mean = sr.mean()
        plt.axvline(x = x_mean, color = color, linestyle = '--', alpha = alpha**.5)
        print(f'{idx}/{len(data)}: label = {label},\ncoverage = {len(sr) - sr.isna().sum()}/{len(sr)} = {1 - sr.isna().sum()/len(sr)}')
        print(sr.describe())
        print()

    plt.legend(loc = loc, prop = fontproperties, framealpha = legend_alpha)
    plt.xticks(fontproperties=fontproperties)
    plt.yticks(fontproperties=fontproperties)
    plt.grid(True)
    plt.xlabel(xlabel=xlabel, fontproperties=fontproperties)
    plt.ylabel(ylabel=ylabel, fontproperties=fontproperties)
    plt.title(title, fontproperties=fontproperties, size = 12)
    plt.show()
    
    print(f'value_range = [{m0}, {m1}]\n')

    return None

# x = pd.Series([1, None, 3, None, 5, 6, None, 8, 9, 10])
# y = pd.Series([1, 2, 3, 4, None, 6, 7, None, 9, 10, None, None])
# plot_corr(x, y, title = 'my_title', xlabel = '123', ylabel = '1233')
